// ==============================================================
//                 ORBITER MODULE: Chariot
//                  Part of the ORBITER SDK
//          Copyright (C) 2002-2004 Martin Schweiger
//                   All rights reserved
//
// Chariot.cpp
// Control module for Chariot vessel class
//
// Notes:
// This is an example for a "minimal" vessel implementation which
// only overloads the clbkSetClassCaps method to define vessel
// capabilities and otherwise uses the default VESSEL class
// behaviour.
// ==============================================================

#define STRICT
#define ORBITER_MODULE
#define ENG 				1	
#define START 				2	
#define STOP 				3	


#include <orbitersdk.h>
#include <cstdio>
#include <cmath>
#include <string>
#include "OrbiterSoundSDK3.h"

#include <stdio.h>
double i = 0;
double i2 = 0.5;
double i2l = 0.5;
double i2r = 0.5;
double i3 = 0;
double t1 = 1;
double i4 = 0;
double i5 = 0;
double i6 = 0;
double st = 0;
double pld = 0;
double cen = 0;
double fb = 0;
double sign = 0;
double pit2 = 200;
double sp = 190;
double i8 = 0;
double main_on = 0;
double retro_on = 0;
int MySoundID;	
double canopy = 0;
double lfdoor = 0;
double lrdoor = 0;
double rfdoor = 0;
double rrdoor = 0;
double rhatch = 0;
double animrobot = 0;
double animMrobot = 0;  
bool ignition = false;
double HATCH_SPEED =.4;
double ROBOT_OPERATING_SPEED= .05;
double robot_proc=0;
double db;
double robot_check;

   
MESHHANDLE chariot = oapiLoadMeshGlobal ("LISCHARIOT");
VISHANDLE MainExternalMeshVisual = 0;

// ==============================================================
// Shuttle-PB class interface
// ==============================================================

class Chariot: public VESSEL2 {
public:
	Chariot (OBJHANDLE hVessel, int flightmodel);
	//	: VESSEL2 (hVessel, flightmodel) {}
	void clbkSetClassCaps (FILEHANDLE cfg);
	void DefineAnimations(void);
	void clbkPostCreation (void);
    void clbkPostStep (double simt, double simdt, double mjd);
   void clbkPreStep (double simt, double simdt, double mjd);
    int  clbkConsumeBufferedKey (DWORD key, bool down, char *kstate);
	void clbkVisualCreated (VISHANDLE vis, int refcount);
	void clbkVisualDestroyed (VISHANDLE vis, int refcount);
	


private:
THRUSTER_HANDLE th_main;
THRUSTER_HANDLE th_retro;
THRUSTER_HANDLE th_rcs[4], th_group[2];
UINT animactive6;
UINT animactive5;
UINT animactive7;
UINT animactive8;
UINT animactive9;
UINT animactive10;
UINT anim_wheel;
UINT anim_wheelleft;
UINT anim_wheelright;
UINT anim_wheelleftrev;
UINT anim_wheelrightrev;
UINT anim_leverleft;
UINT anim_leverright;
UINT anim_lfdoor;
UINT anim_dome;
UINT anim_lrdoor;
UINT anim_rfdoor;
UINT anim_rrdoor;
UINT anim_rhatch;
UINT anim_robot;
UINT animactive11;
UINT animactive12;
 



  
};

Chariot::Chariot (OBJHANDLE hVessel, int flightmodel)
		: VESSEL2 (hVessel, flightmodel)
{
	
	DefineAnimations ();
}


void Chariot::clbkPostCreation (void)
{
MySoundID=ConnectToOrbiterSoundDLL3(GetHandle());
RequestLoadVesselWave3(MySoundID,ENG,"Sound\\Vessel\\Chariot\\chariot-go.wav",BOTHVIEW_FADED_FAR);
RequestLoadVesselWave3(MySoundID,START,"Sound\\Vessel\\Chariot\\chariot-start.wav",BOTHVIEW_FADED_FAR);
SoundOptionOnOff3(MySoundID,PLAYMAINTHRUST,FALSE);
SoundOptionOnOff3(MySoundID,PLAYATTITUDETHRUST,FALSE);

SoundOptionOnOff3(MySoundID,PLAYCOUNTDOWNWHENTAKEOFF,FALSE);

SoundOptionOnOff3(MySoundID,PLAYCABINAIRCONDITIONING,FALSE);

SoundOptionOnOff3(MySoundID,PLAYCABINRANDOMAMBIANCE,FALSE);

SoundOptionOnOff3(MySoundID,PLAYRADIOATC,FALSE);

SoundOptionOnOff3(MySoundID,DISPLAYTIMER,FALSE);

}


// --------------------------------------------------------------
// Set the capabilities of the vessel class
// --------------------------------------------------------------

void Chariot::clbkSetClassCaps (FILEHANDLE cfg)
{

		//THRUSTER_HANDLE th_main;
	//THRUSTER_HANDLE th_hover;

	// physical specs
	SetSize (4);
	SetEmptyMass (100.0);
	SetCW (0.3, 0.3, 0.6, 0.9);
	SetWingAspect (0.1);
	SetWingEffectiveness (0.1);
	SetCrossSections (_V(200,30,30));
	SetRotDrag (_V(0.1,0.1,0.1));
	if (GetFlightModel() >= 1) {
		SetPitchMomentScale (1e-4);
		SetBankMomentScale (1e-4);
	}
	SetPMI (_V(3.47,1.72,2.64));
	SetTrimScale (0.05);
	SetCameraOffset (_V(-.595,1.4,1.4));
	
	SetTouchdownPoints (_V(0,-0.059,1.5), _V(-1.5,-0.059,-1.5), _V(1.5,-0.059,-1.5));
	

const double FUELMASS = 300;
const double ISP = 11e12;
const double MAXMAINTH = 9000;
const double MAXRETROTH = 9000;
const double MAXRCSTH = 3500;
	// propellant resources
    PROPELLANT_HANDLE TANK;
    TANK = CreatePropellantResource (FUELMASS);

	// ***************** thruster definitions *******************
	

	th_main = CreateThruster (_V(0,0,-4.3), _V(0,0,1), MAXRCSTH, TANK, ISP);
	CreateThrusterGroup (&th_main, 1, THGROUP_MAIN);
		
	th_retro = CreateThruster (_V(0,0,4.3), _V(0,0,-1), MAXRCSTH, TANK, ISP);
	CreateThrusterGroup (&th_retro, 1, THGROUP_RETRO);

	th_rcs[0] = CreateThruster (_V( 1,0, 1), _V(0,0,1),MAXRCSTH, TANK, ISP);
	//left front tread
	th_rcs[1] = CreateThruster (_V(-1,0, 1), _V(0,0,1), MAXRCSTH, TANK, ISP);
	// right front tread
	th_rcs[2] = CreateThruster (_V( 1,0,-1), _V(0,0,-1), MAXRCSTH, TANK, ISP);
	//left rear tread
	th_rcs[3] = CreateThruster (_V(-1,0,-1), _V(0,0,-1), MAXRCSTH, TANK, ISP);
    // right rear tread
	th_group[0] = th_rcs[0];
	th_group[1] = th_rcs[3];
	CreateThrusterGroup (th_group, 2, THGROUP_ATT_YAWLEFT);

	th_group[0] = th_rcs[1];
	th_group[1] = th_rcs[2];
	CreateThrusterGroup (th_group, 2, THGROUP_ATT_YAWRIGHT);
			
	// visual specs
	SetMeshVisibilityMode( InsertMesh (chariot,0), MESHVIS_ALWAYS );


}

// ==============================================================
// API callback interface
// ==============================================================

void Chariot::clbkPostStep(double simt, double simdt, double mjd)
{

double vsAirSpeed = GetAirspeed();	
double fDeltadoor = HATCH_SPEED* simdt;
i3 = GetControlSurfaceLevel(AIRCTRL_RUDDER);
//sprintf(oapiDebugString(),"mouth_t %2.9f mouth %2.3f ",mouth_t,mouth);	
VECTOR3 speedvec; // first we define our vector which will recieve the data
oapiGetFocusShipAirspeedVector(&speedvec);//then we retrieve its x,y,z

if (GroundContact()==true && GetAirspeed() >13 && (speedvec.z>0))
{AddForce(_V(0,0,-5e3),_V(0,0,0));}

if (GroundContact()==true && GetAirspeed() >13 && (speedvec.z<0))
{AddForce(_V(0,0,5e3),_V(0,0,0));}
 //Anim Wheels Forward
 
 if (GetThrusterLevel (th_main) >0 && i3 == 0)
 {
    
    if (i > 1)i=0;
    
    if (vsAirSpeed > 0.15) {
	    
	    if (vsAirSpeed > 55) vsAirSpeed = 55;

    
    if (i > 1)i=0;
    i = i + (vsAirSpeed /300);
     
    if (vsAirSpeed ==0) i=0;
    
   // sprintf(oapiDebugString(),"anim %2.2f", i );
    
	SetAnimation (anim_wheelright, i);
    SetAnimation (anim_wheelleft, i);
    
    main_on = 1;
    retro_on = 0;
}

}

//Anim Wheels Backward

if (GetThrusterLevel (th_retro) >0 && i3 == 0)

{
    
    if (i < 0)i=1;
    
    if (vsAirSpeed > 0.15) {
	    
	    if (vsAirSpeed > 55) vsAirSpeed = 55;

    
    if (i < 0)i=1;
    i = i - (vsAirSpeed /300);
     
    if (vsAirSpeed ==0) i=1;
    
   // sprintf(oapiDebugString(),"anim %2.2f", i );
    
	SetAnimation (anim_wheelright, i);
    SetAnimation (anim_wheelleft, i);
    
     main_on = 0;
    retro_on = 1;
    
}


}


//Anim Wheels Forward and Backward when moving without power

if (GetThrusterLevel (th_main)  == 0 && GetThrusterLevel (th_retro) == 0 && vsAirSpeed > 0.15 && i3 == 0)

{
    
	
	if (main_on == 0 && retro_on == 1)
	
	{
    if (i < 0)i=1;
    
    if (vsAirSpeed > 0.15) {
	    
	    if (vsAirSpeed > 55) vsAirSpeed = 55;

    
    if (i < 0)i=1;
    i = i - (vsAirSpeed /300);
     
    if (vsAirSpeed ==0) i=1;
    
   // sprintf(oapiDebugString(),"anim %2.2f", i );
    
	SetAnimation (anim_wheelright, i);
    SetAnimation (anim_wheelleft, i);
    
     
    
}
}

if (main_on == 1 && retro_on == 0)
	
	{
    if (i > 1)i=0;
    
    if (vsAirSpeed > 0.15) {
	    
	    if (vsAirSpeed > 55) vsAirSpeed = 55;

    
    if (i >  1)i=0;
    i = i + (vsAirSpeed /300);
     
    if (vsAirSpeed ==0) i=1;
    
   // sprintf(oapiDebugString(),"anim %2.2f", i );
    
	SetAnimation (anim_wheelright, i);
    SetAnimation (anim_wheelleft, i);
    
    
    
}
}


}




//Anim Wheels Opposite Forward and Backward when turning

// Left turn
if (i3 < -0.15)

{
    
    if (i < 0)i=1;
    
    
	    if (vsAirSpeed > 55) vsAirSpeed = 55;

    
    if (i < 0)i=1;
    i = i - (0.01 + (vsAirSpeed /300));
     
    if (vsAirSpeed ==0) i=1;
    
   // sprintf(oapiDebugString(),"anim %2.2f", i2 );
    
	SetAnimation (anim_wheelrightrev, i);
    SetAnimation (anim_wheelleft, i);
    
  
}

// Right turn
if (i3 > 0.15)

{
    
    if (i < 0)i=1;
    
    
	    
	    if (vsAirSpeed > 55) vsAirSpeed = 55;

    
    if (i < 0)i=1;
    i = i - (0.01 + (vsAirSpeed /300));
     
    if (vsAirSpeed ==0) i=1;
    
 
    
	SetAnimation (anim_wheelright, i);
    SetAnimation (anim_wheelleftrev, i);
    
   
    
}

    
    //Anim Levers
    //sprintf(oapiDebugString(),"anim %2.2f", ignition );
    // Forward Lever animation
    
    if (GetThrusterLevel (th_main) >0 && i3 == 0)
   //Forward Thrust applied not turning, move levers back 
    {
	    i2=i2 - 0.01;
	    if (i2 < .25) i2=.25;
	    
	    SetAnimation (anim_leverleft, i2);
	    SetAnimation (anim_leverright, i2);
	    
    }
	
    
       
	    if (GetThrusterLevel (th_main) == 0 && i2 < 0.5 && i3 == 0)
    
    {
	    i2=i2 + 0.01;
	    if (i2 > 0.5) i2= 0.5;
	    // no thrust, return levers to neutral
	    
	    SetAnimation (anim_leverleft, i2);
	    SetAnimation (anim_leverright, i2);
	    
    }
    
    
    if (GetThrusterLevel (th_main) == 0 && GetThrusterLevel (th_retro) ==0 && i2 == 0.5 && i3 == 0)
    
    {
	    i2 = 0.5;
	   
	  //no thrust, lever neutral  
	    
	    SetAnimation (anim_leverleft, i2);
	    SetAnimation (anim_leverright, i2);
	    
    }
    
    // Reverse Lever animation
    
    if (GetThrusterLevel (th_retro) >0 && i3 == 0 )
    
    {
	    i2=i2 + 0.01;
	    if (i2 > .75) i2= .75;
	    
	    SetAnimation (anim_leverleft, i2);
	    SetAnimation (anim_leverright, i2);
	    
    }
	
    //retro thrust move lever forward
       
	    if (GetThrusterLevel (th_retro) == 0 && i2 > 0.5 && i3 == 0)
    
    {
	    i2=i2 - 0.01;
	    if (i2 < 0.5) i2= 0.5;
	    
	 //no retro thrust return levers to neutral
	    
	    SetAnimation (anim_leverleft, i2);
	    SetAnimation (anim_leverright, i2);
	    
    }
    
    //Turn Lever animation
    
   
    if ( i3 > 0)
    
    {
	    i2= 0.5;
	    i2l = i2l - 0.05;
	    if (i2l < 0) i2l= 0;
	    
	    i2r = i2r + 0.05;
	    if (i2r > 1) i2r= 1;
	    
	    SetAnimation (anim_leverleft, i2l);
	    SetAnimation (anim_leverright, i2r);
	    
    }
    
    if ( i3 == 0 && i2l < 0.5)
    
    {
	    i2l = i2l + 0.05;
	    if (i2l > 0.5) i2l= 0.5;
	    
	    i2r = i2r - 0.05;
	    if (i2r < 0.5) i2r= 0.5;
	    
	    SetAnimation (anim_leverleft, i2l);
	    SetAnimation (anim_leverright, i2r);
	    
    }
    
    
    if ( i3 < 0)
    
    {
	    i2= 0.5;
	    i2l = i2l + 0.05;
	    if (i2l > 1) i2l= 1;
	    
	    i2r = i2r - 0.05;
	    if (i2r < 0) i2r= 0;
	    
	    SetAnimation (anim_leverleft, i2l);
	    SetAnimation (anim_leverright, i2r);
	    
    }
    
    if ( i3 == 0 && i2l > 0.5)
    
    {
	    i2l = i2l - 0.05;
	    if (i2l < 0.5) i2l= 0.5;
	    
	    i2r = i2r + 0.05;
	    if (i2r > 0.5) i2r= 0.5;
	    
	    SetAnimation (anim_leverleft, i2l);
	    SetAnimation (anim_leverright, i2r);
	    
    }
    
 
    
    
    
    // Canopy Animations


   if( animactive5 ) {
		if( animactive5 == 1 ) {
			canopy += fDeltadoor;
			if( canopy > 1.0 ) {
				canopy = 1.0;
				animactive5 = 0;
			}
		}
		else  {
			canopy -= fDeltadoor;
			if( canopy < 0.0 ) {
				canopy = 0.0;
				animactive5 = 0;
			}
		}
		SetAnimation( anim_dome, canopy );
	}
//lfdoor
if( animactive6 ) {
        if( animactive6 == 1 ) {
            lfdoor += fDeltadoor;
            if( lfdoor > 1.0 ) {
                lfdoor = 1.0;
                animactive6 = 0;
            }
        }
        else  {
            lfdoor -= fDeltadoor;
            if( lfdoor < 0.0 ) {
                lfdoor = 0.0;
                animactive6 = 0;
            }
        }
        SetAnimation( anim_lfdoor, lfdoor );
    }
//rfdoor
if( animactive7 ) {
        if( animactive7 == 1 ) {
            rfdoor += fDeltadoor;
            if( rfdoor > 1.0 ) {
                rfdoor = 1.0;
                animactive7 = 0;
            }
        }
        else  {
            rfdoor -= fDeltadoor;
            if( rfdoor < 0.0 ) {
                rfdoor = 0.0;
                animactive7 = 0;
            }
        }
        SetAnimation( anim_rfdoor, rfdoor );
    }
    //lfdoor
if( animactive6 ) {
        if( animactive6 == 1 ) {
            lfdoor += fDeltadoor;
            if( lfdoor > 1.0 ) {
                lfdoor = 1.0;
                animactive6 = 0;
            }
        }
        else  {
            lfdoor -= fDeltadoor;
            if( lfdoor < 0.0 ) {
                lfdoor = 0.0;
                animactive6 = 0;
            }
        }
        SetAnimation( anim_lfdoor, lfdoor );
    }
//rfdoor
if( animactive7 ) {
        if( animactive7 == 1 ) {
            rfdoor += fDeltadoor;
            if( rfdoor > 1.0 ) {
                rfdoor = 1.0;
                animactive7 = 0;
            }
        }
        else  {
            rfdoor -= fDeltadoor;
            if( rfdoor < 0.0 ) {
                rfdoor = 0.0;
                animactive7 = 0;
            }
        }
        SetAnimation( anim_rfdoor, rfdoor );
    }        
    //lfdoor
if( animactive8 ) {
        if( animactive8 == 1 ) {
            lrdoor += fDeltadoor;
            if( lrdoor > 1.0 ) {
                lrdoor = 1.0;
                animactive8 = 0;
            }
        }
        else  {
            lrdoor -= fDeltadoor;
            if( lrdoor < 0.0 ) {
                lrdoor = 0.0;
                animactive8 = 0;
            }
        }
        SetAnimation( anim_lrdoor, lrdoor );
    }
//rfdoor
if( animactive9 ) {
        if( animactive9 == 1 ) {
            rrdoor += fDeltadoor;
            if( rrdoor > 1.0 ) {
                rrdoor = 1.0;
                animactive9 = 0;
            }
        }
        else  {
            rrdoor -= fDeltadoor;
            if( rrdoor < 0.0 ) {
                rrdoor = 0.0;
                animactive9 = 0;
            }
        }
        SetAnimation( anim_rrdoor, rrdoor );
    }        
//rfdoor
if( animactive10 ) {
        if( animactive10 == 1 ) {
            rhatch += fDeltadoor;
            if( rhatch > 1.0 ) {
                rhatch = 1.0;
                animactive10 = 0;
            }
        }
        else  {
            rhatch -= fDeltadoor;
            if( rhatch < 0.0 ) {
                rhatch = 0.0;
                animactive10 = 0;
            }
        }
        SetAnimation( anim_rhatch, rhatch );
    }

	

	
	
	    	    
// engine sound
 double pit = (vsAirSpeed * pit2) + 11025;
 double sp = (190 +((vsAirSpeed)*5));
 if (sp>=225)sp=225;

if (1 == ignition){	 
	{PlayVesselWave3(MySoundID,ENG,LOOP,sp,pit);
}
// Only turn on the engine sound if it is not already running AND if velocity is >= 10 meters-per-second;
// otherwise, only turn OFF the engine sound if it is running AND if velocity is < 10 meters-per-second.
if (!IsPlaying3(MySoundID, ENG) && (vsAirSpeed >= 0))
{
    // turn on the engine sound
    PlayVesselWave3(MySoundID,ENG,LOOP,190);
}
else if (IsPlaying3(MySoundID, ENG) && (vsAirSpeed < 0))
{
    // turn off the engine sound
    StopVesselWave3(MySoundID, ENG);
}
}



// -------------------- added ------------------------------------

if (ignition == 0){
	
		SetThrusterGroupLevel(th_main,0);
		
		SetThrusterLevel(th_main,0);
		
		SetThrusterMax0 (th_main ,0);
		
		SetThrusterGroupLevel(th_retro,0);
		
		SetThrusterLevel(th_retro,0);
		
		SetThrusterMax0 (th_retro ,0);
		
		
		SetThrusterLevel (th_rcs[0], 0);
		SetThrusterMax0 (th_rcs[0] ,0);
		
        SetThrusterLevel (th_rcs[1], 0);
        SetThrusterMax0 (th_rcs[1] ,0);
        
        SetThrusterLevel (th_rcs[2], 0);
        SetThrusterMax0 (th_rcs[2] ,0);
        
        SetThrusterLevel (th_rcs[3], 0);
        SetThrusterMax0 (th_rcs[3] ,0);
        


	
}

  if (ignition == 1){
		
		
		SetThrusterMax0 (th_main ,5000);
		
		
		SetThrusterMax0 (th_retro ,5000);
		
		SetThrusterMax0 (th_rcs[0] ,2500);
		
		SetThrusterMax0 (th_rcs[1] ,2500);
		
		SetThrusterMax0 (th_rcs[2] ,2500);
		
		SetThrusterMax0 (th_rcs[3] ,2500);

	
}

//------------------------------------------------------------







}



void Chariot::DefineAnimations(void)
{      
	
	// changed ******
	
  // Levers
   static UINT LeGrp1[2] = {25,198};
   static MGROUP_ROTATE lever1 (0,LeGrp1, 2, _V(-.634,.824,1.948), _V(1,0,0), (float)(45*RAD));
   static UINT LeGrp2[2] = {26,199};
   static MGROUP_ROTATE lever2 (0,LeGrp2, 2, _V(-.482,.824,1.948), _V(1,0,0), (float)(45*RAD));
      
   anim_leverleft = CreateAnimation (0);
      
   AddAnimationComponent  (anim_leverleft, 0, 1, &lever1);
   
   anim_leverright = CreateAnimation (0);
   
   AddAnimationComponent  (anim_leverright, 0, 1, &lever2);   
   
   
   //Wheels

   static UINT WGrp1[1] = {2};
   static MGROUP_ROTATE wheel1 (0,WGrp1, 1, _V(.891,.329,1.642), _V(1,0,0), (float)(t1*360*RAD));
   
   static UINT WGrp2[1] = {3};
   static MGROUP_ROTATE wheel2 (0,WGrp2, 1, _V(.891,.329,1.091), _V(1,0,0), (float)(t1*360*RAD));
   
   static UINT WGrp3[1] = {4};
   static MGROUP_ROTATE wheel3 (0,WGrp3, 1, _V(.891,.329,.522), _V(1,0,0), (float)(t1*360*RAD));
   
   static UINT WGrp4[1] = {5};
   static MGROUP_ROTATE wheel4 (0,WGrp4, 1, _V(.891,.329,-.041), _V(1,0,0), (float)(t1*360*RAD));
   
   static UINT WGrp5[1] = {6};
   static MGROUP_ROTATE wheel5 (0,WGrp5, 1, _V(.891,.329,-.575), _V(1,0,0), (float)(t1*360*RAD));
   
   static UINT WGrp6[2] = {0,303};
   static MGROUP_ROTATE wheel6 (0,WGrp6, 2, _V(.976,.449,-1.153), _V(1,0,0), (float)(t1*360*RAD));
  
   static UINT WGrp7[1] = {7};
   static MGROUP_ROTATE wheel7 (0,WGrp7, 1, _V(-.901,.329,1.642), _V(1,0,0), (float)(t1*360*RAD));
   
   static UINT WGrp8[1] = {8};
   static MGROUP_ROTATE wheel8 (0,WGrp8, 1, _V(-.901,.329,1.091), _V(1,0,0), (float)(t1*360*RAD));
   
   static UINT WGrp9[1] = {9};
   static MGROUP_ROTATE wheel9 (0,WGrp9, 1, _V(-.901,.329,.522), _V(1,0,0), (float)(t1*360*RAD));
   
   static UINT WGrp10[1] = {10};
   static MGROUP_ROTATE wheel10 (0,WGrp10, 1, _V(-.901,.329,-.041), _V(1,0,0), (float)(t1*360*RAD));
   
   static UINT WGrp11[1] = {11};
   static MGROUP_ROTATE wheel11 (0,WGrp11, 1, _V(-.901,.329,-.575), _V(1,0,0), (float)(t1*360*RAD));
   
   static UINT WGrp12[2] = {1,324};
   static MGROUP_ROTATE wheel12 (0,WGrp12, 2, _V(-.997,.449,-1.153), _V(1,0,0), (float)(t1*360*RAD));    

   static UINT WGrp13[1] = {2};
   static MGROUP_ROTATE wheel13(0,WGrp13, 1, _V(.891,.329,1.642), _V(1,0,0), (float)(-360*RAD));
   
   static UINT WGrp14[1] = {3};
   static MGROUP_ROTATE wheel14 (0,WGrp14, 1, _V(.891,.329,1.091), _V(1,0,0), (float)(-360*RAD));
   
   static UINT WGrp15[1] = {4};
   static MGROUP_ROTATE wheel15 (0,WGrp15, 1, _V(.891,.329,.522), _V(1,0,0), (float)(-360*RAD));
   
   static UINT WGrp16[1] = {5};
   static MGROUP_ROTATE wheel16 (0,WGrp16, 1, _V(.891,.329,-.041), _V(1,0,0), (float)(-360*RAD));
   
   static UINT WGrp17[1] = {6};
   static MGROUP_ROTATE wheel17 (0,WGrp17, 1, _V(.891,.329,-.575), _V(1,0,0), (float)(-360*RAD));
   
   static UINT WGrp18[2] = {0,303};
   static MGROUP_ROTATE wheel18 (0,WGrp18, 2, _V(.976,.448,-1.153), _V(1,0,0), (float)(-360*RAD));
  
   static UINT WGrp19[1] = {7};
   static MGROUP_ROTATE wheel19 (0,WGrp19, 1, _V(-.901,.329,1.642), _V(1,0,0), (float)(-360*RAD));
   
   static UINT WGrp20[1] = {8};
   static MGROUP_ROTATE wheel20 (0,WGrp20, 1, _V(-.901,.329,1.091), _V(1,0,0), (float)(-360*RAD));
   
   static UINT WGrp21[1] = {9};
   static MGROUP_ROTATE wheel21 (0,WGrp21, 1, _V(-.901,.329,.522), _V(1,0,0), (float)(-360*RAD));
   
   static UINT WGrp22[1] = {10};
   static MGROUP_ROTATE wheel22 (0,WGrp22, 1, _V(-.901,.329,-.041), _V(1,0,0), (float)(-360*RAD));
   
   static UINT WGrp23[1] = {11};
   static MGROUP_ROTATE wheel23 (0,WGrp23, 1, _V(-.901,.329,-.575), _V(1,0,0), (float)(-360*RAD));
   
   static UINT WGrp24[2] = {1,324};
   static MGROUP_ROTATE wheel24 (0,WGrp24, 2, _V(-.997,.449,-1.153), _V(1,0,0), (float)(-360*RAD));
   
    anim_wheelright = CreateAnimation (0);
   
   AddAnimationComponent  (anim_wheelright, 0, 1, &wheel7);
   AddAnimationComponent  (anim_wheelright, 0, 1, &wheel8);
   AddAnimationComponent  (anim_wheelright, 0, 1, &wheel9);
   AddAnimationComponent  (anim_wheelright, 0, 1, &wheel10);
   AddAnimationComponent  (anim_wheelright, 0, 1, &wheel11);
   AddAnimationComponent  (anim_wheelright, 0, 1, &wheel12);
   
 anim_wheelleft = CreateAnimation (0);
    
   AddAnimationComponent  (anim_wheelleft, 0, 1, &wheel1);
   AddAnimationComponent  (anim_wheelleft, 0, 1, &wheel2);
   AddAnimationComponent  (anim_wheelleft, 0, 1, &wheel3);
   AddAnimationComponent  (anim_wheelleft, 0, 1, &wheel4);
   AddAnimationComponent  (anim_wheelleft, 0, 1, &wheel5);
   AddAnimationComponent  (anim_wheelleft, 0, 1, &wheel6);
   
   anim_wheelrightrev = CreateAnimation (0);
   
   AddAnimationComponent  (anim_wheelrightrev, 0, 1, &wheel19);
   AddAnimationComponent  (anim_wheelrightrev, 0, 1, &wheel20);
   AddAnimationComponent  (anim_wheelrightrev, 0, 1, &wheel21);
   AddAnimationComponent  (anim_wheelrightrev, 0, 1, &wheel22);
   AddAnimationComponent  (anim_wheelrightrev, 0, 1, &wheel23);
   AddAnimationComponent  (anim_wheelrightrev, 0, 1, &wheel24);
   
   anim_wheelleftrev = CreateAnimation (0);
    
   AddAnimationComponent  (anim_wheelleftrev, 0, 1, &wheel13);
   AddAnimationComponent  (anim_wheelleftrev, 0, 1, &wheel14);
   AddAnimationComponent  (anim_wheelleftrev, 0, 1, &wheel15);
   AddAnimationComponent  (anim_wheelleftrev, 0, 1, &wheel16);
   AddAnimationComponent  (anim_wheelleftrev, 0, 1, &wheel17);
   AddAnimationComponent  (anim_wheelleftrev, 0, 1, &wheel18);
   
   
   
   //Canopy anim
   
   static UINT DGrp1[2] = {381,217};
   
   static MGROUP_TRANSLATE dome (0,DGrp1, 2, _V(0,0,-.85)); 
   
   
   anim_dome = CreateAnimation (0);
   
   AddAnimationComponent  (anim_dome, 0, 1, &dome);

     //Robot anim
   
  static UINT DGrp3[19] = {22,23,256,241,325,334,341,361,362,363,364,366,367,368,369,370,372,373,382};
   
 static MGROUP_ROTATE robot (0,DGrp3, 19, _V(.568,0,-1.08), _V(0,1,0), (float)(2*PI));
  
   
  anim_robot = CreateAnimation (0);
   
   AddAnimationComponent  (anim_robot, 0, 1, &robot);

    // lfdoor
 static UINT HGrp2[7] = {186,215,246,270,271,359,378};
   static MGROUP_ROTATE lfhatch (0,HGrp2, 7, _V(-1.213,1.55,1.130), _V(0,1,0), (float)(160*RAD)); 
   
anim_lfdoor = CreateAnimation (0); 
AddAnimationComponent  (anim_lfdoor, 0, 1, &lfhatch);   

//rfdoor
static UINT HGrp3[7] = {183,211,243,265,266,356,376};
   static MGROUP_ROTATE rfdoor (0,HGrp3, 7, _V(1.198,1.55,1.130), _V(0,1,0), (float)(-160*RAD)); 
   
anim_rfdoor = CreateAnimation (0); 
AddAnimationComponent  (anim_rfdoor, 0, 1, &rfdoor);

// lrdoor
static UINT HGrp4[7] = {185,214,245,263,269,377,358};
   static MGROUP_ROTATE lrdoor (0,HGrp4, 7, _V(-1.213,1.67,-1.134), _V(0,1,0), (float)(-160*RAD)); 
   
anim_lrdoor = CreateAnimation (0); 
AddAnimationComponent  (anim_lrdoor, 0, 1, &lrdoor); 

//rr door
static UINT HGrp5[7] = {184,212,244,267,268,357,375};
   static MGROUP_ROTATE rrdoor (0,HGrp5, 7, _V(1.198,1.67,-1.134), _V(0,1,0), (float)(160*RAD)); 
   
anim_rrdoor = CreateAnimation (0); 
AddAnimationComponent  (anim_rrdoor, 0, 1, &rrdoor); 
// rhatch
static UINT HGrp1[3] = {379,264,247};
   static MGROUP_ROTATE rhatch (0,HGrp1, 3, _V(0,2.112,-1.721), _V(1,0,0), (float)(160*RAD)); 
 
anim_rhatch = CreateAnimation (0); 
AddAnimationComponent  (anim_rhatch, 0, 1, &rhatch);    
}


// --------------------------------------------------------------
// Keyboard interface handler (buffered key events)
// --------------------------------------------------------------
int Chariot::clbkConsumeBufferedKey (DWORD key, bool down, char *kstate)
{
	if (!down) return 0; // only process keydown events

	// if (KEYMOD_SHIFT (kstate)) {

		switch (key) {
		
			
			case OAPI_KEY_1: // dome
		
		if( canopy > 0.5 )
					animactive5 = 2;
				else
					animactive5 = 1;
					return 1;
			case OAPI_KEY_2: // lfdoor
		
		if( lfdoor > 0.5 )
					animactive6 = 2;
				else
					animactive6 = 1;
			return 1;
			
				case OAPI_KEY_3: // rfdoor
				if( rfdoor > 0.5 )
					animactive7 = 2;
				else
					animactive7= 1;
		
			return 1;
				case OAPI_KEY_4: // lrdoor
		
		if( lrdoor > 0.5 )
					animactive8 = 2;
				else
					animactive8 = 1;
						return 1;
				case OAPI_KEY_5: // rrdoor
		
		if( rrdoor > 0.5 )
					animactive9 = 2;
				else
					animactive9 = 1;
						return 1;
				case OAPI_KEY_6: // rhatch
		
		if( rhatch > 0.5 )
					animactive10 = 2;
				else
					animactive10 = 1;
						return 1;
     						
	case OAPI_KEY_K: // ignition
       if(!ignition)
{
//do something unless ignition is true
PlayVesselWave3(MySoundID,START,NOLOOP,255,22100);
ignition=true;
}
      return 1;
case OAPI_KEY_7: // rotate+
        if (robot_check == 0) robot_check = 1;
        else if (robot_check == 1) robot_check = 0;
        else if (robot_check == 2) robot_check = 1;
        return 1;
    case OAPI_KEY_8: // rotate-
        if (robot_check == 0) robot_check = 2;
        else if (robot_check == 2) robot_check = 0;
        else if (robot_check == 1) robot_check = 2;
        return 1;
	
			
	
    } 
	return 0;
}
// ==============================================================
// Visual Created
// ==============================================================
void Chariot::clbkVisualCreated (VISHANDLE vis, int refcount)
{   
	MainExternalMeshVisual = GetMesh(vis,0);
}
// ==============================================================
// Visual destroyed
// ==============================================================
void Chariot::clbkVisualDestroyed (VISHANDLE vis, int refcount)
{
	MainExternalMeshVisual = 0;	
}


// --------------------------------------------------------------
// Vessel initialisation
// --------------------------------------------------------------
DLLCLBK VESSEL *ovcInit (OBJHANDLE hvessel, int flightmodel)
{
	return new Chariot (hvessel, flightmodel);
}

// --------------------------------------------------------------
// Vessel cleanup
// --------------------------------------------------------------
//DLLCLBK void ovcExit (VESSEL *vessel)
//{
//if (vessel) delete (Chariot*)vessel;
//}

void Chariot::clbkPreStep(double SimT, double SimDT, double MJD) {
   



    
   db = (SimDT * ROBOT_OPERATING_SPEED);

    if (robot_proc > 1) robot_proc = (robot_proc - 1);
    else if (robot_proc < 0) robot_proc = (robot_proc + 1);

   if (robot_check == 1)robot_proc = (robot_proc + db);
    else if (robot_check == 2)robot_proc = (robot_proc - db);
    else if (robot_check == 0)robot_proc = robot_proc;

    SetAnimation (anim_robot, robot_proc);
}